/*****************************************************************************
 *                                                                           
 *  Copyright (c) 2006 by SANTEC/TUDOR www.santec.tudor.lu                   
 *                                                                           
 *                                                                           
 *  This library is free software; you can redistribute it and/or modify it  
 *  under the terms of the GNU Lesser General Public License as published    
 *  by the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                      
 *                                                                           
 *  This software is distributed in the hope that it will be useful, but     
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
 *  Lesser General Public License for more details.                          
 *                                                                           
 *  You should have received a copy of the GNU Lesser General Public         
 *  License along with this library; if not, write to the Free Software      
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
 *                                                                           
 *****************************************************************************/

import ij.ImagePlus;
import ij.ImageStack;
import ij.io.SaveDialog;
import ij.plugin.filter.PlugInFilter;
import ij.process.ImageProcessor;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;

import lu.tudor.santec.dicom.anonymizer.DicomAnonymizer;
import lu.tudor.santec.dicom.anonymizer.SimpleAnonPanel;
import lu.tudor.santec.dicom.exporter.DicomExporter;

import org.dcm4che2.data.DicomObject;

import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * ImageJ Plugin that saves dicom images to the filesystem
 * @author Johannes Hermen johannes.hermen(at)tudor.lu
 *
 */
public class Save_As_Dicom implements PlugInFilter, ActionListener {

	private static final long serialVersionUID = 1L;
	private ImagePlus ip;
	private String path;
	private DicomExporter de;
	private SimpleAnonPanel anonPanel = new SimpleAnonPanel();
	private JDialog anonDialog;
	private JButton cancelButton;
	private JButton okButton;

	public int setup(String path, ImagePlus ip) {
	    this.ip = ip;
	    this.path = path;
	    
	    anonDialog = new JDialog();
	    anonDialog.setModal(true);
	    anonDialog.setLayout(new BorderLayout());
	    anonDialog.add(anonPanel, BorderLayout.CENTER);
	    anonPanel.setRemovePatient(false);
	    anonPanel.setRemoveInstitution(false);
	    anonPanel.setRemoveManufacturer(false);
	    
	    ButtonBarBuilder bb = new ButtonBarBuilder();
		
		bb.addGlue();
		
		cancelButton = new JButton("cancel");
		cancelButton.addActionListener(this);
		bb.addGridded(cancelButton);
		
		bb.addRelatedGap();
		
		okButton = new JButton("run");
		okButton.addActionListener(this);
		bb.addGridded(okButton);
		
		JPanel buttonPanel = bb.getPanel();
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
		buttonPanel.setOpaque(false);
		anonDialog.add(buttonPanel, BorderLayout.SOUTH);
		anonDialog.pack();
	    
	    return DOES_8G+DOES_16+DOES_RGB;
	}

	public void run(ImageProcessor arg0) {
	    try {
	    
		System.out.println("run: " + "Save_As_Dicom Path: " + path);
		File f = null;
		if (path != null && path != "") {
			f = new File(path);
		} else {
			SaveDialog sd = new SaveDialog("filename","exported" + ip.getBitDepth(),".dcm");
			try {
				f = new File(sd.getDirectory() + File.separatorChar + sd.getFileName());
			} catch (Exception e) {
				e.printStackTrace();
				return;
			}
		}
		
		de = new DicomExporter();
		anonDialog.setVisible(true);
		
		if (path != null && path != "") {
		    
		}
		
		if (ip.getStackSize() > 1) {
			ImageStack is = ip.getStack();
			for (int j = 1; j < is.getSize()+1; j++) {
				String fileName = f.getAbsolutePath();
				try {
					fileName = fileName.split(".")[0] + "_" + j + fileName.split(".")[1]; 
				} catch (Exception e) {
					fileName = fileName + "_" + j;
				}
				File f2 = new File(fileName);
				ImagePlus imagePlus = new ImagePlus(f2.getName(), is.getProcessor(j));
				System.out.println("Saving file: " + f2.getAbsolutePath());
				exportFile(imagePlus, f2);
			}
		} else {
		    	exportFile(ip, f);
			System.out.println("Saving file: " + f.getAbsolutePath());
		}

        	} catch (Exception e) {
        	    e.printStackTrace();
        	}
	}

	public void exportFile(ImagePlus ip, File f) throws Exception{
	    	DicomObject dObj = de.createHeader(ip, true, true, true);
	    	 if (anonPanel.isRemovePatient())
			DicomAnonymizer.removePatientInfo(dObj, anonPanel.getNewPatientID(), anonPanel.getNewPatientName());
		    
		    if (anonPanel.isRemoveInstitution())
			DicomAnonymizer.removeInstitutionInfo(dObj, anonPanel.getNewInstitutionName());
		    
		    if (anonPanel.isRemoveManufacturer())
			DicomAnonymizer.removeManufacturerInfo(dObj);
		de.writeImage(dObj, ip, f);
	}

	public void actionPerformed(ActionEvent e) {
	    if (e.getSource().equals(this.okButton)) {
		anonDialog.setVisible(false);
	    } 
	}
}
