package lu.tudor.santec.dicom.gui.header;
/*****************************************************************************
 *                                                                           
 *  Copyright (c) 2006 by SANTEC/TUDOR www.santec.tudor.lu                   
 *                                                                           
 *                                                                           
 *  This library is free software; you can redistribute it and/or modify it  
 *  under the terms of the GNU Lesser General Public License as published    
 *  by the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                      
 *                                                                           
 *  This software is distributed in the hope that it will be useful, but     
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
 *  Lesser General Public License for more details.                          
 *                                                                           
 *  You should have received a copy of the GNU Lesser General Public         
 *  License along with this library; if not, write to the Free Software      
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
 *                                                                           
 *****************************************************************************/

import ij.ImagePlus;

import java.io.File;
import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Logger;

import org.dcm4che2.data.DicomElement;
import org.dcm4che2.data.DicomObject;
import org.dcm4che2.data.SpecificCharacterSet;
import org.dcm4che2.data.Tag;
import org.dcm4che2.io.DicomInputStream;
import org.dcm4che2.io.StopTagInputHandler;
import org.dcm4che2.util.StringUtils;
import org.dcm4che2.util.TagUtils;

/**
 * @author Johannes Hermen johannes.hermen(at)tudor.lu
 *
 */
public class DicomHeader {

	private String rawHeader = "";

	public Vector<HeaderTag> headerTags = new Vector<HeaderTag>();

	private DicomObject dicomObject;

	/**
	 * @return the dicomObject
	 */
	public DicomObject getDicomObject() {
	    return dicomObject;
	}

	/**
	 * @param dicomObject the dicomObject to set
	 */
	public void setDicomObject(DicomObject dicomObject) {
	    this.dicomObject = dicomObject;
	}

	/**
	 * @return the dCM4CHE
	 */
	public boolean isDCM4CHE() {
	    return dCM4CHE;
	}

	/**
	 * @param dcm4che the dCM4CHE to set
	 */
	public void setDCM4CHE(boolean dcm4che) {
	    dCM4CHE = dcm4che;
	}


	private boolean dCM4CHE;

	// the logger for this class
	private static Logger logger = Logger
			.getLogger("lu.tudor.santec.dicom.gui.viewer.DicomHeader");

	/**
	 * Please use DicomHeader(File f) instead
	 * @param rawHeader
	 */
	public DicomHeader(String rawHeader) {
		if( rawHeader != null) {
			this.rawHeader = rawHeader;
			this.parseHeader();
		}
	}
	
	public DicomHeader(File f) {
		if( f.canRead()) {
		    readHeaderDCM4CHE(f);
		}
	}
	
	@SuppressWarnings("unchecked")
	private void readHeaderFromFile(File f) {
		try {
			this.headerTags = new DicomDecoder().getDicomHeader(f);
		} catch (Exception e) {
			try {
			    System.out.println("error reading DICOM Header from File, trying to use ImageJ functions...");
			    e.printStackTrace();
				ImagePlus ipl = new ImagePlus(f.getAbsolutePath());
				this.rawHeader = ipl.getProperty("Info").toString();
				ipl = null;
				this.parseHeader();
			} catch (Exception ee) {
				logger.finest("File " + f +  " has no DICOM Header !!!");
			}
		}
	}

	@SuppressWarnings("unchecked")
	private void readHeaderDCM4CHE(File f) {
		try {
		    DicomInputStream in = new DicomInputStream(f);
	            in.setHandler(new StopTagInputHandler(Tag.PixelData));
	            DicomObject dcmobj =  in.readDicomObject();
	            in.close();
	            this.dicomObject = dcmobj;
	            this.dCM4CHE = true;
	            for (Iterator iterator = dcmobj.iterator(); iterator.hasNext();) {
			DicomElement element = (DicomElement) iterator.next();
			HeaderTag ht = new HeaderTag(element, dcmobj);
			headerTags.add(ht);
		    }
		} catch (Exception e) {
		    e.printStackTrace();
		    logger.finest("File " + f +  " has no DICOM Header !!!");
		}
	}
	
	
	/**
	 * Please use DicomHeader(File f) instead
	 * @param ipl
	 */
	public DicomHeader(ImagePlus ipl) {
	    	if (ipl.getOriginalFileInfo() != null) {
	    	    File f = new File(ipl.getOriginalFileInfo().directory + ipl.getOriginalFileInfo().fileName);
	    	    if (f.canRead() && f.isFile()) {
	    		readHeaderDCM4CHE(f);
	    	    } else {
	    		try {
	    		    if (ipl != null) { 
	    			this.rawHeader = ipl.getProperty("Info").toString();
	    			ipl = null;
	    			this.parseHeader();
	    		    }	
	    		} catch (Exception e) {
	    		    logger.warning("ImagePlus has no DICOM Header !!!");
	    		}	    	    
	    	    }	    	    
	    	} else {
	    		try {
	    		    if (ipl != null) { 
	    			this.rawHeader = ipl.getProperty("Info").toString();
	    			ipl = null;
	    			this.parseHeader();
	    		    }	
	    		} catch (Exception e) {
	    		    logger.warning("ImagePlus has no DICOM Header !!!");
	    		}	    	    
	    	}
	}
	
	public DicomHeader() {
	}

	private void parseHeader() {
		long start = System.currentTimeMillis();
		String[] lineArr = rawHeader.split("\n");
		for (int i = 0; i < lineArr.length; i++) { 
			HeaderTag ht = new HeaderTag(lineArr[i]);
			if (ht.tagNr != null){
				headerTags.add(ht);
			} else if (headerTags.size() > 0) {
				HeaderTag lastTag =(HeaderTag)headerTags.lastElement();
				lastTag.tagValue = lastTag.tagValue + "\n" + ht.tagValue;
			}
			
		}
		logger.fine("Parsing DICOM Header took: "
				+ (System.currentTimeMillis() - start) + " ms, Header has " + headerTags.size() + " elements");
	}
	
	/**
	 * 
	 * @param headerNr e.g. "0018,0050" to get Slice Thickness<br>
	 * or "0008,0102#0054,0220" to get the Coding Scheme Designator after View Code Sequence
	 * @return double
	 * @throws DicomHeaderParseException
	 */
	public double getHeaderDoubleValue(String headerNr) throws DicomHeaderParseException {
		double doubleValue = Double.NaN;
		String value = "";
		try {
			if (headerNr.indexOf('#') > -1 ) {
				String[] tags = headerNr.split("#");
				value = getHeaderValueAfterTag(tags[1], tags[0]).trim();
			} else {
				value = getHeaderTagValue(headerNr).trim();
			}
			doubleValue = Double.parseDouble(value);
		} catch (Exception e) {
			logger.warning("Error: (double) HeaderValue not available! " + headerNr + " cant parse \"" + value + "\"  to double");
			throw new DicomHeaderParseException(headerNr);
		}
		return doubleValue;
	}

	/**
	 * 
	 * @param headerNr e.g. "0018,0050" to get Slice Thickness<br>
	 * or "0008,0102#0054,0220" to get the Coding Scheme Designator after View Code Sequence
	 * @return int
	 * @throws DicomHeaderParseException
	 */
	public int getHeaderIntegerValue(String headerNr) throws DicomHeaderParseException {
		int intValue = (int) Double.NaN;
		String value = "";
		try {
			if (headerNr.indexOf('#') > -1 ) {
				String[] tags = headerNr.split("#");
				value = getHeaderValueAfterTag(tags[1], tags[0]).trim();
			} else {
			    value =  getHeaderTagValue(headerNr).trim();
			}
			intValue = Integer.parseInt(value);
		} catch (Exception e) {
			logger.warning("Error: (int) HeaderValue not available! " + headerNr + " cant parse \"" + value + "\"  to int");
			throw new DicomHeaderParseException(headerNr);
		}
		return intValue;
	}
	
	public String getHeaderStringValue(int headerTag) {
	    return getHeaderStringValue(toTagString(headerTag));
	}
	
	public double getHeaderDoubleValue(int headerTag) throws DicomHeaderParseException {
	    return getHeaderDoubleValue(toTagString(headerTag));
	}
	
	public int getHeaderIntegerValue(int headerTag) throws DicomHeaderParseException {
	    return getHeaderIntegerValue(toTagString(headerTag));
	}
	
	/**
	 * 
	 * @param headerNr e.g. "0018,0050" to get Slice Thickness<br>
	 * or "0008,0102#0054,0220" to get the Coding Scheme Designator after View Code Sequence
	 * @return String
	 * @throws DicomHeaderParseException
	 */
	public String getHeaderStringValue(String headerNr) {
		String value = null;
		if (headerNr.indexOf('#') > -1 ) {
			String[] tags = headerNr.split("#");
			value = getHeaderValueAfterTag(tags[1], tags[0]);
		} else {
			value = getHeaderTagValue(headerNr);
		}
		if (value != null)
			value = value.trim();
		return value;
	}
	
	/**
	 * @deprecated
	 * @param headerNr
	 * @return
	 */
	public String getHeaderValue(String headerNr) {
		if (headerNr.indexOf('#') > -1 ) {
			String[] tags = headerNr.split("#");
			return getHeaderValueAfterTag(tags[1], tags[0]);
		}
			return getHeaderTagValue(headerNr);
	}

	private String getHeaderTagValue(String headerNr) {
		try {
			for (Iterator iter = headerTags.iterator(); iter.hasNext();) {
				HeaderTag ht = (HeaderTag) iter.next();
				if (ht.tagNr.equals(headerNr))
					return ht.tagValue;
			}
			return "";
		} catch (Exception e) {
			logger.info("DICOM Tag: " + headerNr + " not found");
			return "";
		}
	}
	
	public boolean containsHeaderTag(String headerNr) {
		try {
			for (Iterator iter = headerTags.iterator(); iter.hasNext();) {
				HeaderTag ht = (HeaderTag) iter.next();
				if (ht.tagNr.equals(headerNr))
					return true;
			}
			return false;
		} catch (Exception e) {
			logger.info("DICOM Tag: " + headerNr + " not found");
			return false;
		}
	}
	
	public String getHeaderValueAfterTag(String headerNr, String afterTagNr) {
		try {
			boolean found = false;
			for (Iterator iter = headerTags.iterator(); iter.hasNext();) {
				HeaderTag ht = (HeaderTag) iter.next();
				if (ht.tagNr.equals(afterTagNr))
					found = true;
				else if (found && ht.tagNr.equals(headerNr)) {
					return ht.tagValue;
				}
			}
			return "";
		} catch (Exception e) {
			logger.info("DICOM Tag: " + headerNr + " after " + afterTagNr + " not found");
			return "";
		}
	}

	public static String getHeaderName(String headerNr) {
		try {
			String val =  DicomDictionary.getDictionary().getProperty(headerNr.replaceAll(",", ""));
			if (val == null)
				return "";
			return val.substring(2);
		} catch (Exception e) {
			logger.info("DICOM Tag: " + headerNr + " not found");
			return "";
		}
	}
	
	public static String getHeaderFieldType(String headerNr) {
		try {
			String val =  DicomDictionary.getDictionary().getProperty(headerNr.replaceAll(",", ""));
			if (val == null)
				return "";
			return val.substring(0,2);
		} catch (Exception e) {
			logger.info("DICOM Tag: " + headerNr + " not found");
			return "";
		}
	}

	public String getHeader(String headerNr) {
		try {
			for (Iterator iter = headerTags.iterator(); iter.hasNext();) {
				HeaderTag ht = (HeaderTag) iter.next();
				if (ht.tagNr.equals(headerNr))
					return ht.tagName + ": " + ht.tagValue;
			}
			return "";
		} catch (Exception e) {
			logger.info("DICOM Tag: " + headerNr + " not found");
			return "";
		}
	}
	
	public void addHeaderField(String tagNr, String tagName, String tagValue) {
		HeaderTag ht = new HeaderTag(tagNr, tagName, tagValue);
		headerTags.add(ht);
	}
	
	public void addHeaderField(String tagNr, String tagName, String tagValue, String tagType) {
		HeaderTag ht = new HeaderTag(tagNr, tagName, tagValue, tagType);
		headerTags.add(ht);
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
		String oldNr = "";
		for (Iterator it = headerTags.iterator(); it.hasNext(); ) {
		    try {
		    	HeaderTag ht = (HeaderTag) it.next();
				if (! oldNr.split(",")[0].equals(ht.tagNr.split(",")[0])) {
						sb.append("------------------------------------------------------------" + "\n");		
				}
				oldNr = ht.tagNr;
				sb.append(ht.toString() + "\n");
			} catch (Exception e) {
			}
		}
		return sb.toString();
	}


	public static void main(String[] argsv) {
		DicomHeader dh = new DicomHeader(new File("/home/hermenj/im1"));
		System.out.println(dh.toString());
	}

	public boolean isEmpty() {
		if (headerTags.size() != 0) {
			return false;
		} 
		return true;
	}
	
//	public static DicomHeader createDefaultHeader() {
//		Date now = new Date();
//		UIDGenerator uidg = UIDGenerator.getInstance();
//		DateFormat  dateFormatter = new SimpleDateFormat("yyyyMMdd");
//		String dateString =  dateFormatter.format(now);
//		DateFormat  timeFormatter = new SimpleDateFormat("HHmmss");
//		String timeString =  timeFormatter.format(now);		
//		DicomHeader dh = new DicomHeader();
//		String unique = System.currentTimeMillis() + "";
//		
//		String patientName = "Tudor DICOM Export";
//		
//		// 0002 -------------------------------------------------------------------------------
//		dh.addHeaderField("0002,0016","Source Application Entity Title","Tudor DICOM");
//		// 0008 -------------------------------------------------------------------------------		
////		dh.addHeaderField("0008,0016", "SOP Class UID", uidg.createUID());
////		dh.addHeaderField("0008,0018", "SOP Instance UID", uidg.createUID());
//		dh.addHeaderField("0008,0012", "Instance Creation Date", dateString);
//		dh.addHeaderField("0008,0013", "Instance Creation Time", timeString);
//		dh.addHeaderField("0008,0020", "Study Date", dateString);
//		dh.addHeaderField("0008,0021", "Series Date", dateString);
//		dh.addHeaderField("0008,0022", " Acquisition Date", dateString);
//		dh.addHeaderField("0008,0023", "Image Date", dateString);
//		dh.addHeaderField("0008,0030", "Study Time", timeString);
//		dh.addHeaderField("0008,0031", "Series Time", timeString);
//		dh.addHeaderField("0008,0032", "Acquisition Time", timeString);
//		dh.addHeaderField("0008,0033", "Image Time", timeString);
//		dh.addHeaderField("0008,0050", "Accession Number", "");
//		dh.addHeaderField("0008,0060", "Modality", "OT");
//		dh.addHeaderField("0008,0070", " Manufacturer", "Tudor Santec");
//		dh.addHeaderField("0008,0080", "Institution Name", "Tudor Santec");
//		dh.addHeaderField("0008,0090", "Referring Physician's Name", "");
//		dh.addHeaderField("0008,1010", "Station Name", "");
//		dh.addHeaderField("0008,1030", "Study Description", "DICOM Export");
//		dh.addHeaderField("0008,103E", "Series Description", "DICOM Export");
//		dh.addHeaderField("0008,1060","Name of Physician(s) Reading Study", "");
//		dh.addHeaderField("0008,1070", "Operator's Name", "");
//		dh.addHeaderField("0008,1090",  "Manufacturer's Model Name", "Tudor DICOM Tools");
//		// 0010 -------------------------------------------------------------------------------		
//		dh.addHeaderField("0010,0010", "Patient's Name", patientName);
//		dh.addHeaderField("0010,0020", "Patient ID", patientName);
//		dh.addHeaderField("0010,0030", "Patient's Birth Date", "");
//		dh.addHeaderField("0010,0040", "Patient's Sex", "");
//		dh.addHeaderField("0010,1010", "Patient's Age", "000Y");
//		dh.addHeaderField("0010,21B0", "Additional Patient History", "");
//		// 0020 -------------------------------------------------------------------------------		
//		dh.addHeaderField("0020,000D","Study Instance UID", uidg.createUID());
//		dh.addHeaderField("0020,000E","Series Instance UID", uidg.createUID());
//		dh.addHeaderField("0020,0010", "Study ID", unique);
//		dh.addHeaderField("0020,0011", "Series Number", "1");
//		dh.addHeaderField("0020,0012", "Acquisition Number", "1");
//		dh.addHeaderField("0020,0013", "Image Number", "1");
//		return dh;
//	}

	@SuppressWarnings("unchecked")
	public HeaderTag getHeaderTag(String headerNr) {
		try {
			for (Iterator iter = headerTags.iterator(); iter.hasNext();) {
				HeaderTag ht = (HeaderTag) iter.next();
				if (ht.tagNr.equals(headerNr))
					return ht;
			}
			return null;
		} catch (Exception e) {
			logger.info("DICOM Tag: " + headerNr + " not found");
			return null;
		}
	}
	
	public DicomHeader clone() {
	    DicomHeader dh = new DicomHeader();
	    dh.rawHeader = this.rawHeader;
	    for (Iterator iter = headerTags.iterator(); iter.hasNext();) {
		HeaderTag ht = (HeaderTag) iter.next();
		dh.addHeaderField(ht.getTagNr(), ht.getTagName(), ht.getTagValue(), ht.getTagType());
	    }
	    return dh;
	}
	
	
	public DicomHeader getUnifiedHeader() {
	    return DicomHeaderUnifier.getUnifiedHeader(this);
	}
	
	public static String toTagString(int tagNr) {
	    return StringUtils.shortToHex(tagNr >> 16) + 
		',' + StringUtils.shortToHex(tagNr);
	}
	
}

