package lu.tudor.santec.dicom.gui.viewer;
/*****************************************************************************
 *                                                                           
 *  Copyright (c) 2006 by SANTEC/TUDOR www.santec.tudor.lu                   
 *                                                                           
 *                                                                           
 *  This library is free software; you can redistribute it and/or modify it  
 *  under the terms of the GNU Lesser General Public License as published    
 *  by the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                      
 *                                                                           
 *  This software is distributed in the hope that it will be useful, but     
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
 *  Lesser General Public License for more details.                          
 *                                                                           
 *  You should have received a copy of the GNU Lesser General Public         
 *  License along with this library; if not, write to the Free Software      
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
 *                                                                           
 *****************************************************************************/

import ij.ImagePlus;
import ij.gui.Roi;

import java.awt.Color;
import java.awt.Point;
import java.util.Vector;

import lu.tudor.santec.dicom.gui.Icons;

/**
 * @author Johannes Hermen johannes.hermen(at)tudor.lu
 *
 */
public class DicomImageCornerSelector {

	private DicomImagePanel dicomImagePanel;
	private int size;
	private Point[] points = new Point[4];
	
	public DicomImageCornerSelector(DicomImagePanel dip, int sizePixels) {
		
		this.dicomImagePanel = dip;
		this.size = sizePixels;
		
//		 select corners manual
		this.dicomImagePanel.addModeButton(Icons.getIcon(Icons.ACTION_CORNERS), "Select Corners",new ImageListener() {
			Vector<Roi> v = new Vector<Roi>();
			int i = 0;
			public void pixelClicked(ImagePlus ip, int x, int y) {
				int corner = RoiBuilder.CORNER_TOP_LEFT;

				if (x > (ip.getWidth()/2)) {
					if (y > (ip.getHeight()/2)) {
						corner = RoiBuilder.CORNER_BOTTOM_RIGHT;
					} else {
						corner = RoiBuilder.CORNER_TOP_RIGHT;	
					}
				} else {
					if (y > (ip.getHeight()/2)) {
						corner = RoiBuilder.CORNER_BOTTOM_LEFT;
					} else {
						corner = RoiBuilder.CORNER_TOP_LEFT;	
					}
				}				
				if (i >= 4) {
					v.removeAllElements();
					i = 0;
				} else {
					RoiBuilder.createCornerRoi(v,x,y,size, corner, Color.YELLOW);
					points[i] = new Point(x,y);
					i++;
				}
				dicomImagePanel.setDefaultRois(v);
			}
			public void mouseMoved(int x, int y) {				
			}
			public void optionSelected(boolean selected) {
				if (!selected) {
					v.removeAllElements();
				} else {
					points = new Point[4];
				}
			}});
	}

	public Point[] getCorners() {
		return points;
	}
	
	
	
}
