  package lu.tudor.santec.dicom.gui.viewer;
/*****************************************************************************
 *                                                                           
 *  Copyright (c) 2006 by SANTEC/TUDOR www.santec.tudor.lu                   
 *                                                                           
 *                                                                           
 *  This library is free software; you can redistribute it and/or modify it  
 *  under the terms of the GNU Lesser General Public License as published    
 *  by the Free Software Foundation; either version 2 of the License, or     
 *  (at your option) any later version.                                      
 *                                                                           
 *  This software is distributed in the hope that it will be useful, but     
 *  WITHOUT ANY WARRANTY; without even the implied warranty of               
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
 *  Lesser General Public License for more details.                          
 *                                                                           
 *  You should have received a copy of the GNU Lesser General Public         
 *  License along with this library; if not, write to the Free Software      
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
 *                                                                           
 *****************************************************************************/

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.Vector;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import lu.tudor.santec.dicom.anonymizer.DicomAnonymizerGui;
import lu.tudor.santec.dicom.gui.DICOMSettingsPlugin;
import lu.tudor.santec.dicom.gui.DicomFileDialog;
import lu.tudor.santec.dicom.gui.Icons;
import lu.tudor.santec.dicom.gui.MemoryMonitorButton;
import lu.tudor.santec.dicom.gui.header.DicomDiffDialog;
import lu.tudor.santec.i18n.Translatrix;
import lu.tudor.santec.settings.LoggingPlugin;
import lu.tudor.santec.settings.SettingsPanel;

import com.l2fprod.common.swing.JButtonBar;

/**
 * @author Johannes Hermen johannes.hermen(at)tudor.lu
 *
 */
public class Viewer extends JFrame implements	ActionListener, FocusListener, SliceListener{

	public static final String VERSION = "0.8.4";
	
	private Logger logger = Logger.getLogger(Viewer.class.getName());
	
	private static final long serialVersionUID = 1L;

	private JButton showSettings;

	private JButton infoButton;

	private SettingsPanel settings;

	private LoggingPlugin loggingPlugin;

	private DICOMSettingsPlugin dicomPlugin;

	private DicomFileDialog dicomFileDialog;

	private JButton inputButton;

	private JToolBar toolBar;

	private JButton helpButton;
	
	private DicomImagePanel[] imagePanels;
	
	private JToolBar imageToolBar;
	private JPanel imageToolBarPanel = new JPanel(new GridLayout(1,1));

//	private ThumbnailBar thumbnailBar;
	private SeriesBar seriesBar;

//	private StartScriptPlugin startupPlugin;

	private JToolBar settingBar;

	private JToggleButton screen1;

	private JToggleButton screen2v;

	private JToggleButton screen2h;

	private JToggleButton screen4;
	
	private JPanel screenPanel;

	private DicomImagePanel activeScreen;

	private Properties screenSettings = new Properties();

	protected String splitscreen;

	private ActionListener layoutSwitcher;

	private JToggleButton three_monitor;

	private final static String infoMessage = "<html><h1>SANTEC/TUDOR Dicom Viewer " +VERSION+"</h1>"
		+ "<h2>Centre de Recherche Public Henri Tudor</h2>"
		+ "<h3>Centre de Ressources Santec www.santec.tudor.lu<br>" +
				"29, Avenue John F. Kennedy<br>" +
				"L-1855 Luxembourg - Kirchberg<h3>"
			+ "written by Johannes Hermen johannes.hermen@tudor.lu<br>"
			+ "Distributable under LGPL license - (c) 2005 Tudor/Santec<br>" 
			+ "Visit http://projects.santec.lu/optimage for Infos and Updates<br><br>"
			+ "the following libraries were used to build and run this project:<br>"
			+ "<ul>"
			+ "<li><p>dcm4che: A OpenSource DICOM Toolkit<br>Distributable under LGPL license<br>Copyright (c) 2002, 2003 by TIANI MEDGRAPH AG<br>http://sourceforge.net/projects/dcm4che/<br>&nbsp;</p></li>"
			+ "<li><p>imageJ: Image Processing and Analysis in Java<br>Distributable under public-domain<br>Rasband, W.S., ImageJ, U. S. National Institutes of Health<br>http://rsb.info.nih.gov/ij/<br>&nbsp;</p></li>"
			+ "<li><p>FormLayout: Build better screens faster<br>Distributable under BSD Licence<br>Copyright (c) 2003 JGoodies<br>http://www.jgoodies.com/freeware/forms/<br>&nbsp;</p></li>"
			+ "<li><p>L2FProd.com: Common Components (ButtonBar)<br>Distributable under SkinLF License<br>Copyright 2004, l2fprod.com<br>http://www.l2fprod.com/<br>&nbsp;</p></li>"
			+ "</ul>";

	private static final File PROPERTY_FILE = new File("screensettings.properties");

	private static final String MAIN_LOCATION_X = "MAIN_LOCATION_X";

	private static final String MAIN_LOCATION_Y = "MAIN_LOCATION_Y";

	private static final String MAIN_WIDTH = "MAIN_WIDTH";

	private static final String MAIN_HEIGHT = "MAIN_HEIGHT";

	private static final String SPLITSCREEN = "SPLITSCREEN";
	
	JDialog[] dialogs = {
			new JDialog(),
			new JDialog()
			};

	private JButton diffButton;

	private JButton anonButton;
	
	/**
	 * create a new Instance of the Plugin
	 */
	public Viewer(boolean exitOnCLose) {
		super("SANTEC/TUDOR DICOM Viewer " + VERSION);
		if (exitOnCLose)
		    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		else
		    this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				saveSettings();
				if (dicomFileDialog != null && dicomFileDialog.pacsPanel != null) {
					dicomFileDialog.pacsPanel.stopDicomServer();
				}
				dicomFileDialog = null;
				settings = null;
			}
		});
		this.buildPanel();
	}
	
	/**
	 * create a new Instance of the Plugin
	 */
	public Viewer(boolean exitOnCLose, String[] fileNames) {
		super("SANTEC/TUDOR DICOM Viewer " + VERSION);
		if (exitOnCLose)
		    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		else
		    this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				saveSettings();
				if (dicomFileDialog != null && dicomFileDialog.pacsPanel != null) {
					dicomFileDialog.pacsPanel.stopDicomServer();
				}
				dicomFileDialog = null;
				settings = null;
			}
		});
		this.buildPanel();
		
		File[] files = new File[fileNames.length];
		for (int i = 0; i < fileNames.length; i++) {
			logger.info("loading file: " + fileNames[i]);			
			files[i] = new File(fileNames[i]);
		}
		this.setFiles(files);
	}

	private void setFiles(File[] files) {
		seriesBar.addSeries(files);
//		thumbnailBar.setThumbs(files);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new Viewer(true, args);
	}
	
	private void center(JFrame parent, JDialog dialog) {
		Dimension f = parent.getSize();
		int posX = parent.getX();
		int posY = parent.getY();
		dialog.setLocation(posX + (f.width/ 2) - dialog.getSize().width/2 , posY + (f.height / 2) - dialog.getSize().height/2);
	}

	private void buildPanel() {
	
		for (int i = 0; i < dialogs.length; i++) {
			dialogs[i].setTitle("Screen " + (i+1));
			dialogs[i].setLayout(new BorderLayout());
			dialogs[i].setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		}
		
		Translatrix.addBundle("lu.tudor.santec.settings.resources.WidgetResources");
		
		this.setIconImage(Icons.getImage(Icons.DICOM_VIEWER,22));
		this.getContentPane().setLayout(new BorderLayout());
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);


		toolBar = new JToolBar();
		
		settingBar = new JToolBar();
		
		// create the settings
		this.buildSettings();
		this.showSettings = new JButton(Icons.getIcon(Icons.ICON_SETTINGS));
		this.showSettings.setToolTipText("Show Settings Dialog");
		this.showSettings.addActionListener(this);
		
		// create the dicom file chooser
		this.buildInput();
		this.inputButton = this.dicomFileDialog.getToolBarButton();
		this.inputButton.addActionListener(this);
		// add input
		toolBar.add(inputButton);
					
		// create the info
		this.infoButton = new JButton(Icons.getIcon(Icons.ICON_INFO));
		this.infoButton.setToolTipText("Show Info Dialog");
		this.infoButton.addActionListener(this);
		
		// create the helpButton
		this.helpButton = new JButton(Icons.getIcon(Icons.ICON_HELP));
		this.helpButton.setToolTipText("Show Help Dialog");
		this.helpButton.addActionListener(this);
		
		layoutSwitcher = new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				screen1.setSelected(false);
				screen2h.setSelected(false);
				screen2v.setSelected(false);
				screen4.setSelected(false);
				three_monitor.setSelected(false);
				
				screenPanel.removeAll();
				
				splitscreen = e.getActionCommand();
				
				if (e.getActionCommand().equals("screen1")) {
					screen1.setSelected(true);
					screenPanel.setLayout(new GridLayout(1,1));
					createImagePanels(1,800,800, false);
				}else if (e.getActionCommand().equals("screen2h")) {
					screen2h.setSelected(true);
					screenPanel.setLayout(new GridLayout(2,1));
					createImagePanels(2,800,400, false);
				}else if (e.getActionCommand().equals("screen2v")) {
					screen2v.setSelected(true);
					screenPanel.setLayout(new GridLayout(1,2));
					createImagePanels(2,400,800, false);
				}else if (e.getActionCommand().equals("screen4")) {
					screen4.setSelected(true);
					screenPanel.setLayout(new GridLayout(2,2));
					createImagePanels(4,400,400, false);
				} else if (e.getActionCommand().equals("three_monitor")) {
					three_monitor.setSelected(true);
					screenPanel.setLayout(new GridLayout(1,1));
					createImagePanels(2,800,800, true);
				}
				
				screenPanel.validate();
			}
		};
		
		toolBar.addSeparator();
		this.screen1 = new JToggleButton(Icons.getIcon(Icons.SCREEN_1));
		this.screen1.setToolTipText("1 Screen");
		this.screen1.setSelected(true);
		this.screen1.setActionCommand("screen1");
		this.screen1.addActionListener(layoutSwitcher);
		toolBar.add(this.screen1);
		this.screen2v = new JToggleButton(Icons.getIcon(Icons.SCREEN_2V));
		this.screen2v.setToolTipText("2 Screens Vertical");
		this.screen2v.setActionCommand("screen2v");
		this.screen2v.addActionListener(layoutSwitcher);
		toolBar.add(this.screen2v);
		this.screen2h = new JToggleButton(Icons.getIcon(Icons.SCREEN_2H));
		this.screen2h.setToolTipText("2 Screens Horizontal");
		this.screen2h.setActionCommand("screen2h");
		this.screen2h.addActionListener(layoutSwitcher);
		toolBar.add(this.screen2h);
		this.screen4 = new JToggleButton(Icons.getIcon(Icons.SCREEN_4));
		this.screen4.setToolTipText("4 Screens");
		this.screen4.setActionCommand("screen4");
		this.screen4.addActionListener(layoutSwitcher);
		toolBar.add(this.screen4);
		this.three_monitor = new JToggleButton(Icons.getIcon(Icons.THREE_MONITOR));
		this.three_monitor.setToolTipText("3 Monitors");
		this.three_monitor.setActionCommand("three_monitor");
		this.three_monitor.addActionListener(layoutSwitcher);
		toolBar.add(this.three_monitor);
		
		toolBar.addSeparator();
		this.diffButton = new  JButton(Icons.getIcon(Icons.ICON_DIFF));
		this.diffButton.setToolTipText("Show DICOM-Diff Dialog");
		this.diffButton.addActionListener(this);
		toolBar.add(diffButton);
		
		toolBar.addSeparator();
		// create the anonymizer button
		this.anonButton = new JButton(Icons.getIcon(Icons.ICON_ANON));
		this.anonButton.setToolTipText("Anonymize DICOM Files");
		this.anonButton.addActionListener(this);
		toolBar.add(anonButton);


		// add settings
		settingBar.add(showSettings);

		// add info
		settingBar.add(infoButton);
		
		// add help
		settingBar.add(helpButton);

		settingBar.addSeparator();

		// add memory monitor
		settingBar.add(new MemoryMonitorButton(true, true));

		JPanel toolBarPanel = new JPanel(new BorderLayout());
		
		toolBarPanel.add(toolBar, BorderLayout.CENTER);
		toolBarPanel.add(settingBar, BorderLayout.EAST);
		this.getContentPane().add(toolBarPanel, BorderLayout.NORTH);

		
		this.screenPanel=new JPanel(new GridLayout(1,1));
		this.layoutSwitcher.actionPerformed(new ActionEvent(
				this, 0, "screen1"));

//		thumbnailBar = new ThumbnailBar(JButtonBar.VERTICAL,64, this.activeScreen.parentPanel);
		seriesBar = new SeriesBar(JButtonBar.VERTICAL,96, this.activeScreen.parentPanel, false);
		
		this.getContentPane().add(this.screenPanel,BorderLayout.CENTER);
		this.getContentPane().add(imageToolBarPanel,BorderLayout.WEST);
//		this.getContentPane().add(thumbnailBar,BorderLayout.EAST);
		this.getContentPane().add(seriesBar,BorderLayout.EAST);
		this.pack();
		
		
//		// testing hermenj
//		if (System.getProperty("user.name").equals("hermenj")) {
//			this.setLocation(1280, 0);
//			this.setSize(1024, 768);
//		}
		

		loadSettings();
		
		if (! splitscreen.equals("three_monitor"))
			this.setExtendedState(JFrame.MAXIMIZED_BOTH);
		
		this.setVisible(true);
	}

	private void createImagePanels(int anz, int width, int height, boolean external) {
		imagePanels = new DicomImagePanel[anz];
		for (int i = 0; i < anz; i++) {
			DicomImagePanel imageArea = new DicomImagePanel(10, 10, true, false,false);
			imagePanels[i] = imageArea.createBigArea();
			imagePanels[i].setImageSize(width,height);
			imagePanels[i].addFocusListener(this);
			imagePanels[i].setShowDicomInfos(true);
			new DicomImageMeasurement(imageArea);
			new DicomImageAngleMeasurement(imageArea);
			new DicomImageInverter(imageArea);
			new DicomImageInterpolator(imageArea);
			
//			new DicomImageCornerSelector(imageArea,100);
//			new DicomImageRotator(imageArea);
//			new DicomImageStackSlider(imageArea);
			imagePanels[i].setSliceListener(this);
			
			if (! external) {
				this.screenPanel.add(imagePanels[i]);
			} else {
				try {
					dialogs[i].getContentPane().remove(0);					
				} catch (Exception e) {
				}
				dialogs[i].getContentPane().add(imagePanels[i]);
				dialogs[i].setSize(800,800);
				dialogs[i].setVisible(true);
			}
		}
		
		if (! external) {
			for (int i = 0; i < dialogs.length; i++) {
				dialogs[i].setVisible(false);
			}
		} else {
			this.setSize(267, this.getWidth());
			this.setExtendedState(JFrame.MAXIMIZED_VERT);
		}
		
		this.activeScreen = imagePanels[0];
		try {
			imageToolBarPanel.removeAll();	
		} catch (Exception e) {
		}
		this.imageToolBar = activeScreen.createToolBar();
		imageToolBarPanel.add(this.imageToolBar);
		imageToolBarPanel.validate();

		if (this.seriesBar != null) {
			this.seriesBar.setImagePanel(this.activeScreen.parentPanel);
		}
	}
	
	
	
	/**
	 * create the settings Dialog
	 */
	private void buildSettings() {
		this.dicomPlugin = new DICOMSettingsPlugin("dicom");
		this.loggingPlugin = new LoggingPlugin("logging");
//		this.startupPlugin = new StartScriptPlugin("startscript","dicomviewer","Dicom_Viewer.app/Contents/Resources/Java/lu.tudor.santec.dicom.jar");
		this.settings = new SettingsPanel(this);
		this.settings.addPlugin(dicomPlugin);
		this.settings.addPlugin(loggingPlugin);
//		this.settings.addPlugin(startupPlugin);
		this.settings.setSettingsFile(new File("settings.xml"));
		this.settings.loadSettings();
	}

	/**
	 * create the Input Dialog
	 */
	private void buildInput() {
		dicomFileDialog = new DicomFileDialog(this, settings, dicomPlugin);
	}

	/**
	 * create the Info Dialog
	 */
	private void buildInfo() {
		JOptionPane.showMessageDialog(this, infoMessage,
				"Info: Tudor DICOM Viewer", JOptionPane.INFORMATION_MESSAGE,
				Icons.getIcon(Icons.ICON_INFO));

	}
	
	/**
	 * create the help Dialog
	 */
	private void buildHelp() {
		JDialog dialog = new JDialog(this, "Help: Simple DICOM Viewer");
		JEditorPane editorPane = new JEditorPane();
		editorPane.setEditable(false);
		try {
			editorPane.setPage(Icons.class.getResource("resources/dicomviewer.html"));
			JScrollPane editorScrollPane = new JScrollPane(editorPane);
			dialog.getContentPane().add(editorScrollPane);
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setSize(600, 600);
			dialog.setVisible(true);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(this.showSettings)) {
			this.settings.setSize(650,650);
			center(this, this.settings);
			this.settings.setVisible(true);
		} else if (e.getSource().equals(this.inputButton)) {
			this.importFiles();
		} else if (e.getSource().equals(this.infoButton)) {
			this.buildInfo();
		}else if (e.getSource().equals(this.helpButton)) {
			this.buildHelp();
		} else if (e.getSource().equals(this.diffButton)) {
			new DicomDiffDialog(this.dicomFileDialog);
		} else if (e.getSource().equals(this.anonButton))  {
		    new DicomAnonymizerGui(false);
		}
	}

	/**
	 * import the selected Files into ImageJ
	 */
	public void importFiles() {
		
		// use Thread while loading Images
		new Thread() {
			@SuppressWarnings("unchecked")
			public void run() {

				Vector filterTags = null;
				
				if (JFileChooser.APPROVE_OPTION == dicomFileDialog
						.showNewDialog(Viewer.this, filterTags)) {
					
					if (dicomFileDialog.singleFileSelected) {
						File[] fArr = {dicomFileDialog
								.getSelectedFile()};
//						thumbnailBar.setThumbs(fArr);
						seriesBar.addSeries(fArr);
					} else {
//						thumbnailBar.setThumbs(dicomFileDialog
//								.getSelectedFiles());
						seriesBar.addSeries(dicomFileDialog
								.getSelectedFiles());
					}
				}
			}

		}.start();
	}

    /**
     * sets the Mousecursor of the MainFrame to a WaitCursor and Back
     *
     * @param on true=waitcursor false=normalcursor
     */
    public void setWaitCursor(boolean on)
    {
        if (on) {
            getGlassPane().setVisible(true); 
            getGlassPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            
        } else {
            getGlassPane().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            getGlassPane().setVisible(false); 
        }
    }

	public void focusGained(FocusEvent e) {
		try {
			imageToolBarPanel.removeAll();	
		} catch (Exception e1) {
		}
		this.activeScreen = (DicomImagePanel) e.getSource();
		this.imageToolBar = activeScreen.createToolBar();
		imageToolBarPanel.add(this.imageToolBar);
		imageToolBarPanel.validate();
		
		try {
			this.seriesBar.setCurrentSeries((Series)this.activeScreen.parentPanel.series);
		} catch (Exception e2) {
//			System.out.println("series is null!!!");
		}

		this.seriesBar.setImagePanel(this.activeScreen.parentPanel);
		
	}

	public void focusLost(FocusEvent e) {
	}

	public void nextSlice() {
		this.seriesBar.showNextImage();
	}

	public void previousSlice() {
		this.seriesBar.showPreviousImage();
	}
	
	private void loadSettings() {
		try {
			screenSettings.load(new FileInputStream(PROPERTY_FILE));
			
			this.setLocation(
					Integer.parseInt(screenSettings.getProperty(MAIN_LOCATION_X)),
					Integer.parseInt(screenSettings.getProperty(MAIN_LOCATION_Y)));
			
			this.setSize(
					Integer.parseInt(screenSettings.getProperty(MAIN_WIDTH)),
					Integer.parseInt(screenSettings.getProperty(MAIN_HEIGHT)));
			
			this.layoutSwitcher.actionPerformed(new ActionEvent(
					this, 
					0,
					screenSettings.getProperty(SPLITSCREEN)));
			
			for (int i = 0; i < dialogs.length; i++) {
				dialogs[i].setLocation(
						Integer.parseInt(screenSettings.getProperty("DIALOG_" + i  + "_LOCATION_X")),
						Integer.parseInt(screenSettings.getProperty("DIALOG_" + i  + "_LOCATION_Y")));
				dialogs[i].setSize(
						Integer.parseInt(screenSettings.getProperty("DIALOG_" + i  + "_WIDTH")),
						Integer.parseInt(screenSettings.getProperty("DIALOG_" + i  + "_HEIGHT")));
			}
			
		} catch (Exception e) {
		    System.out.println(e.getLocalizedMessage());
		}
	}
	
	private void saveSettings() {
		try {
			screenSettings.put(MAIN_LOCATION_X, this.getLocation().x + "");
			screenSettings.put(MAIN_LOCATION_Y, this.getLocation().y + "");
			screenSettings.put(MAIN_WIDTH, this.getWidth() + "");
			screenSettings.put(MAIN_HEIGHT, this.getHeight() + "");
			screenSettings.put(SPLITSCREEN, splitscreen);
			
			for (int i = 0; i < dialogs.length; i++) {
				screenSettings.put("DIALOG_" + i  + "_LOCATION_X", dialogs[i].getLocation().x + "");
				screenSettings.put("DIALOG_" + i  + "_LOCATION_Y", dialogs[i].getLocation().y + "");
				screenSettings.put("DIALOG_" + i  + "_WIDTH", dialogs[i].getWidth() + "");
				screenSettings.put("DIALOG_" + i  + "_HEIGHT", dialogs[i].getHeight() + "");
			}
			
			screenSettings.store(new FileOutputStream(PROPERTY_FILE), null);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
}
